// Storage for our point set: 
ArrayList<Point2d> points;

// The convex hull: 
Polygon hull = null;

// A reference to any selected point: 
Point2d selectedPoint = null;

// Used for perturbing the points a bit to gaurantee 
// no 3 collinear points. 
final static Random rand = new Random();
final static double eps = 1e-10;

double perturb() {
  return (rand.nextDouble() - 0.5)*eps;
}

void update() {
  // If a point is selected, reset its vertices
  if (selectedPoint != null) {
    selectedPoint.x = mouseX + perturb();
    selectedPoint.y = mouseY + perturb();
    
    // Obtain the convex hull
    updateHull();
  }
}

void updateHull() {
  hull = null;
  switch (chAlgorithm) {
    case INCREMENTAL: hull = convexHullIncremental(points); break;
    case GIFTWRAPPING: hull = convexHullGiftWrapping(points); break;
    case GRAHAMSCAN: hull = convexHullGrahamScan(points); break;
  }
}

void draw() {
  
  // Update objects based on user input: 
  update();
  
  // Clear the screen: 
  background(255);
  
  // Draw the convex hull
  if (hull != null && hull.firstVertex != null) {
    stroke(0);
    Vertex v = hull.firstVertex;
    int i = 0; 
    do {
      line((int)v.coord.x, (int)v.coord.y, (int)v.next.coord.x, (int)v.next.coord.y);
      v = v.next;
      i++;
    } while (v != hull.firstVertex && i < 1000);
    if (i == 1000) System.out.println("Error");
  }
  
  // Draw the point set
  int i = 0;
  for (Point2d p : points) {
    if (p == selectedPoint) {
      fill(255, 0, 0);
    } else {
      fill(255);
    }
    stroke(0);
    ellipse((int)Math.round(p.x), (int)Math.round(p.y), 8, 8);
    fill(0);
    text(" " + i++, (int)p.x, (int)p.y);
  }
  
  // Draw a message to the user: 
  textSize(11);
  fill(128);
  text("Hold the CTRL key down when selecting vertices to add/remove.", 10, height - 22);
  text("Current Algorithm: " + chAlgorithm.getName() + ".    Type 'a' to cycle algorithms.", 10, height - 10);
}

void setup() {
  // Setup the window: 
  size(640, 480);
  smooth(8);
  
  // Initialize the point set: 
  points = new ArrayList<Point2d>();
  points.add(new Point2d(300, 400));
  points.add(new Point2d(400, 400));
  points.add(new Point2d(350, 300));
  points.add(new Point2d(300, 300));
  
  // Initialize the hull:
  updateHull();
}