import java.util.Comparator; 
import java.util.Random;

class Point2d {
  double x, y; 
  
  public Point2d(int x, int y) {
    this.x = x + perturb();
    this.y = y + perturb();
  }
  
  public String toString() {
    return "(" + x + " , " + y + ")";
  }
  
  /**
   * Computes the square distance from this 
   * point to that one. Better to use this for comparing
   * distances than to use the actual distance because
   * this avoids the square-root function and rounding errors. 
   * 
   * @param that The point to compare to. 
   * @return The squared distance from this to that. 
   */
  double distSqTo(Point2d that) {
      double dx = that.x - this.x;
      double dy = that.y - this.y;
      return dx*dx + dy*dy;
  }
}

/** 
 * A Comparator for sorting a list of Point2d objects by x-coordinate. 
 *
 * use with Collections.sort(S) to sort a list S of type List<Point2d>
 * to sort the list by x-coordinate. (remember to import java.util.Collections)
 *
 */
class Point2dXComparator implements Comparator<Point2d> {
  public int compare(Point2d p, Point2d q) {
    return p.x < q.x ? -1 : p.x > q.x ? +1 : 
           p.y < q.y ? -1 : p.y > q.y ? -1 : 0;
  }
}