/**
 * @return true if A-B-C constitutes a left-hand turn; false otherwise.
 */
boolean isLeftHandTurn(Point2d A, Point2d B, Point2d C) {
  return areaOfParallelogram(A, B, C) < 0;
}

/**
 * @return true if A-B-C constitutes a left-hand turn or collinear; false otherwise.
 */
boolean isLeftHandTurnOrOn(Point2d A, Point2d B, Point2d C) {
  return areaOfParallelogram(A, B, C) <= 0;
}

/**
 * @return true if the set of points A-B-C is collinear; false otherwise.
 */
boolean isCollinear(Point2d A, Point2d B, Point2d C) {
  return areaOfParallelogram(A, B, C) == 0;
}

/**
 * @return true if A-B-C constitutes a right-hand turn or collinear; false otherwise.
 */
boolean isRightHandTurn(Point2d A, Point2d B, Point2d C) {
  return areaOfParallelogram(A, B, C) > 0;
}

/**
 * @return true if A-B-C constitutes a right-hand turn; false otherwise.
 */
boolean isRightHandTurnOrOn(Point2d A, Point2d B, Point2d C) {
  return areaOfParallelogram(A, B, C) >= 0;
}

boolean isSameSide(Point2d A, Point2d B, Point2d C, Point2d D) {
  return (isLeftHandTurn(A, B, C) && isLeftHandTurn(A, B, D)) || 
         (isRightHandTurn(A, B, C) && isRightHandTurn(A, B, D));
}

/**
 * @return the signed area of the parallelogram with consecutive vertices B, A, C
 */
double areaOfParallelogram(Point2d A, Point2d B, Point2d C) {
    return (B.x - A.x) * (C.y - A.y) - (C.x - A.x) * (B.y - A.y);
}

/**
 * @return the signed area of the triangle with corners A, B, C.
 */
double areaOfTriangle(Point2d A, Point2d B, Point2d C) {
    return 0.5 * areaOfParallelogram(A, B, C);
}