/////////////////////////////////////////////////////////////
// Keyboard and Mouse Input
/////////////////////////////////////////////////////////////

// An enum of the various algorithm types used to store which 
// algorithm th user wants.
static enum CHAlgorithmChoice {
  INCREMENTAL("Incremental"), GIFTWRAPPING("Gift Wrapping"), GRAHAMSCAN("Graham Scan");
  private String name; 
  public String getName() { return name; }
  private CHAlgorithmChoice(String name) { this.name = name; }
  private static CHAlgorithmChoice[] vals = values();
  public CHAlgorithmChoice next()
  {
      return vals[(this.ordinal()+1) % vals.length];
  }
};

// The current choice of algorithm
CHAlgorithmChoice chAlgorithm = CHAlgorithmChoice.INCREMENTAL;

// If true, mouse clicks add/delete vertices rather than select them
boolean addDeleteMode = false; 

void keyPressed() {
  // If the CTRL key is pressed, switch to add/delete mode. 
  if (key == CODED && keyCode == CONTROL) {
    addDeleteMode = true;
  } else if (key == 'a') {
    chAlgorithm = chAlgorithm.next();
    updateHull();
  }
}

void keyReleased() {
  // If the CTRL key is released, switch out of add/delete mode. 
  if (key == CODED && keyCode == CONTROL) {
    addDeleteMode = false;
  }
}

void mousePressed() {
  // Create a Point2d for the current click: 
  Point2d mousePoint = new Point2d(mouseX, mouseY);

  // Loop over all vertices and store a reference to one that is close enough: 
  for (Point2d p : points) {
    if (p.distSqTo(mousePoint) < 64) {
      selectedPoint = p;
      break;
    }
  }
  
  // If we are in add/delete mode, we will add a new point if the mouse click
  // was not sufficiently close to any existing point, otherwise, we'll delete
  // the selected point: 
  if (addDeleteMode) {
    if (selectedPoint == null) {
      points.add(mousePoint);
      selectedPoint = mousePoint;
    } else {
      points.remove(selectedPoint);
    }
  }
}

void mouseReleased() {
  // Reset selected point on release. 
  selectedPoint = null;
}